from __future__ import annotations

from typing import Dict

from lxml import etree as et

from eml.resources import Resource


# TODO: Implement Protocol Resource (https://eml.ecoinformatics.org/schema/eml_xsd#eml_protocol)
class EMLProtocol(Resource):
    """
    EML Protocol Resource.

    Other Parameters
    ----------------
    **kwargs : :class:`eml.resources.resource.Resource` parameters.
        The parameters of every type of Resource.
    """
    def __init__(self, **kwargs) -> None:
        # To avoid initialization
        # super().__init__(referencing=True, **kwargs)
        raise NotImplementedError("Protocol EML class not implemented yet")

    @classmethod
    def get_referrer(cls, element: et.Element, nmap: Dict) -> EMLProtocol:
        """
        Generate an EML Protocol referencing another EML Protocol.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse with references another protocol.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLProtocol
            Object parsed that reference another protocol.
        """
        pass

    @classmethod
    def get_no_referrer(cls, element: et.Element, nmap: Dict) -> EMLProtocol:
        """
        Generate an EML protocol that do not reference another protocol.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLProtocol
            Object parsed.
        """
        pass

    def to_element(self) -> et.Element:
        """
        Generate an XML `Element` from this object

        Returns
        -------
        lxml.etree.Element
            XML `Element` from this object

        Raises
        ------
        NotImplementedError
            Protocol functionality not implemented yet
        """
        raise NotImplementedError("Protocol functionality not implemented yet")



from typing import Iterable, TypeVar

T = TypeVar('T')


def iterate_with_bar(iterator: Iterable[T], **kwargs) -> T:
    try:
        from tqdm import tqdm
        return tqdm(iterator, **kwargs)
    except ImportError:
        return iterator


class OptionalTqdm:
    def __init__(self, **kwargs) -> None:
        try:
            from tqdm import tqdm
            self.__tqdm__ = tqdm(**kwargs)
        except ImportError:
            self.__tqdm__ = None
        return

    def set_descriptor(self, **kwargs) -> None:
        if self.__tqdm__ is not None:
            self.__tqdm__.set_description(**kwargs)
        return

    def update(self, **kwargs) -> None:
        if self.__tqdm__ is not None:
            self.__tqdm__.update(**kwargs)
        return

    def close(self) -> None:
        if self.__tqdm__ is not None:
            self.__tqdm__.close()
        return

    def set_postfix(self, **kwargs) -> None:
        if self.__tqdm__ is not None:
            self.__tqdm__.set_postfix(**kwargs)
        return

    def reset(self, **kwargs) -> None:
        if self.__tqdm__ is not None:
            self.__tqdm__.reset(**kwargs)
        return



from __future__ import annotations

from typing import Dict, List

from lxml import etree as et

from eml.resources import Resource
from eml.types import ExtensionString, I18nString, ResponsibleParty


class EMLDataset(Resource):
    """
    Dataset represents the base type for the dataset element on an EML document.

    Parameters
    ----------
    contact: List[ResponsibleParty]
        The contact for this dataset.

    Other Parameters
    ----------------
    **kwargs : :class:`eml.resources.resource.Resource` parameters.
        The parameters of every type of Resource.
    """
    PRINCIPAL_TAG = "dataset"

    def __init__(
            self,
            contact: List[ResponsibleParty] = None,
            **kwargs
    ) -> None:
        super().__init__(**kwargs)
        self.__contact__ = list()
        if self.referencing:
            return
        if contact is None or len(contact) == 0:
            raise ValueError("No contact provided")
        self.__contact__.extend(contact)
        return

    @property
    def contacts(self) -> List[ResponsibleParty]:
        """List[ResponsibleParty]: The contact for this dataset."""
        return self.__contact__

    # TODO: Implement other attributes (https://eml.ecoinformatics.org/schema/eml_xsd#eml_dataset)

    @classmethod
    def get_referrer(cls, element: et.Element, nmap: Dict) -> EMLDataset:
        """
        Generate an EML Dataset referencing another EML Dataset.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse with references another dataset.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLDataset
            Object parsed that reference another dataset.
        """
        kwargs = super().parse_kwargs(element, nmap)
        return EMLDataset(**kwargs)

    @classmethod
    def get_no_referrer(cls, element: et.Element, nmap: Dict) -> EMLDataset:
        """
        Generate an EML Dataset that do not reference another dataset.

        Parameters
        ----------
        element : lxml.etree.Element
            XML element to parse.
        nmap : Dict
            Namespace.

        Returns
        -------
        EMLDataset
            Object parsed.
        """
        kwargs = super().parse_kwargs(element, nmap)
        contact = list()
        for contact_elem in element.findall("contact", nmap):
            contact.append(ResponsibleParty.parse(contact_elem, nmap))
        return EMLDataset(
            contact=contact,
            **kwargs
        )

    def to_element(self) -> et.Element:
        """
        Generates an XML `Element` from this instance

        Returns
        -------
        `lxml.etree.Element`
            XML `Element` from this instance
        """
        dataset = super().to_element()
        for contact in self.contacts:
            contact.set_tag("contact")
            dataset.append(contact.to_element())
        return dataset



import unittest
import datetime as dt
from typing import Union

from xml_common.utils import type_to_sql


class TestTypeSQL(unittest.TestCase):
    def test_int(self):
        self.assertEqual("INTEGER", type_to_sql(int), "Error converting int.")

    def test_float(self):
        self.assertEqual("REAL", type_to_sql(float), "Error converting float.")

    def test_str(self):
        self.assertEqual("VARCHAR", type_to_sql(str), "Error converting str.")

    def test_bool(self):
        self.assertEqual("BOOLEAN", type_to_sql(bool), "Error converting bool.")

    def test_datetime(self):
        self.assertEqual("DATETIME", type_to_sql(dt.datetime), "Error converting datatime.")

    def test_bytes(self):
        self.assertEqual("BLOB", type_to_sql(bytes), "Error converting byte.")

    def test_object(self):
        self.assertEqual("VARCHAR", type_to_sql(Union[str, int]), "Error converting union.")
        class A:
            pass
        self.assertEqual("VARCHAR", type_to_sql(A), "Error converting object.")


if __name__ == '__main__':
    unittest.main()



import unittest

from eml.resources import EMLProtocol
from test_xml.test_xml import TestXML


class TestEMLProtocol(TestXML):
    def test_not_implemented(self):
        self.assertRaises(NotImplementedError, EMLProtocol)


if __name__ == '__main__':
    unittest.main()



import os.path
import unittest

from dwca.classes import OutsideClass
from test_dwca_classes.test_outside_class_common import TestOutsideCommon

PATH = os.path.abspath(os.path.dirname(__file__))


class TestOutside(TestOutsideCommon):
    def setUp(self) -> None:
        self.outside_xml, self.text = self.read_xml(os.path.join(PATH, os.pardir, "example_data", "meta.xml"))
        return

    def test_name(self):
        outside_class = OutsideClass.from_string(self.text)
        self.assertEqual("SpeciesProfile", outside_class.name, "Name parsed incorrectly.")

    def test_parse(self):
        super().__test_parse__()

    def test_none(self):
        super().__test_none__()

    def test_merge(self):
        super().__test_merge__()

    def test_missing_field(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
</core>
            """
        )

    def test_field_twice(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            """
<core>
    <files>
      <location>missing.txt</location>
    </files>
    <id index="0"/>
    <field index="0" term=""/>
    <field index="1" term=""/>
    <field index="3" term=""/>
    <field index="2" term=""/>
    <field index="3" term=""/>
    <field index="4" term=""/>
</core>
            """
        )

    def test_parse_invalid(self):
        self.assertRaises(
            AssertionError,
            OutsideClass.from_string,
            '<extension />'
        )
        self.assertRaises(
            KeyError,
            OutsideClass.from_string,
            """
<core-id>
    <field index="0" term=""/>
</core-id>
            """
        )

    def test_set_primary_key(self):
        self.__test_set_primary_key__()

    def test_sql_table(self):
        self.__test_sql_table__()


if __name__ == '__main__':
    unittest.main()
